#import "Calculator.h"

#undef INHIBIT_OPEN_SAVE

#if defined(INHIBIT_OPEN_SAVE)
#define selector_load	@selector(inhibitedPanel:)
#define selector_save	@selector(inhibitedPanel:)
#define selector_saveAs @selector(inhibitedPanel:)
#else
#define selector_load	@selector(load:)
#define selector_save	@selector(save:)
#define selector_saveAs @selector(saveAs:)
#endif




@implementation Calculator

	static float lineHeight=20.0;
	static float label_left=10.0;
	static float label_width=270.0;
	static float field_right=360.0;

	static float field_left;
	static float field_width;

	static SEL   selector_none=@selector(none:);

	- init
	{
		[NSApp setDelegate:self];
		filename=nil;

		[NSApp setMainMenu:[self makeMenu]];

		field_left=label_left+label_width+5.0;
		field_width=field_right-field_left;
		[self buildWindow];
		[calculatorWindow setDelegate:self];
		[calculatorWindow makeKeyAndOrderFront:self];

		return self;
	}//init;
	


	-(void)addLabelNamed:(NSString*)name at:(float)y
	{
		NSTextField* label;
		label=[[NSTextField alloc]
			initWithFrame:NSMakeRect(label_left,y,label_width,lineHeight)];
		[label setStringValue:name];
		[label setAlignment:NSRightTextAlignment];
		[label setEditable:NO];
		[label setBezeled:NO];
		[label setBordered:NO];
		[label setBackgroundColor:[NSColor lightGrayColor]];
		[label setAutoresizingMask:NSViewMaxXMargin|NSViewMinYMargin];
		[[calculatorWindow contentView]addSubview:label];
	}//addLabelNamed:at:;


	-(NSTextField*)addTextFieldNamed:(NSString*)name at:(float*)y
				action:(SEL)action
	{
		NSTextField* field;
		(*y)-=lineHeight+4.0;

		[self addLabelNamed:name at:*y];

		field=[[NSTextField alloc]
			initWithFrame:NSMakeRect(field_left,*y,field_width,lineHeight)];
		[field setIntValue:0];
		[field setAlignment:NSRightTextAlignment];
		[field setAutoresizingMask:NSViewWidthSizable|NSViewMinYMargin];
		[[calculatorWindow contentView]addSubview:field];

		if(action==selector_none){
			[field setEditable:NO];
			[field setBezeled:NO];
			[field setBordered:NO];
			[field setBackgroundColor:[NSColor lightGrayColor]];
		}else{
			[field setEnabled:YES];
			[field setTarget:self];
			[field setAction:action];
		}
		return(field);
	}//addTextFieldNamed:at:action:;


	-(NSScroller*)addScrollerNamed:(NSString*)name at:(float*)y
				action:(SEL)action
	{
		NSTextField* label;
		NSScroller*	 scroller;
		(*y)-=lineHeight+4.0;

		[self addLabelNamed:name at:*y];

		scroller=[[NSScroller alloc]
			initWithFrame:NSMakeRect(field_left,*y,field_width,lineHeight)];
		[scroller setAutoresizingMask:NSViewWidthSizable|NSViewMinYMargin];
		[[calculatorWindow contentView]addSubview:scroller];

		if(action!=selector_none){
			[scroller setEnabled:YES];
			[scroller setTarget:self];
			[scroller setAction:action];
		}
		return(scroller);
	}//addScrollerNamed:at:action:;


	-(NSMatrix*)addMatrixNamed:(NSString*)name at:(float*)y
				action:(SEL)action
				choices:(NSArray/*of NSString*/*)choices
	{
		NSTextField*	label;
		NSMatrix*	 	matrix;
		NSButtonCell*	buttonPrototype;
		unsigned int count=[choices count];
		unsigned int i;

		(*y)-=lineHeight+4;

		[self addLabelNamed:name at:*y];

		(*y)-=(count-1)*(lineHeight+2);

		buttonPrototype=[[[NSButtonCell alloc]initTextCell:@""]autorelease];
		[buttonPrototype setButtonType:NSRadioButton];
		[buttonPrototype setBezeled:NO];
		[buttonPrototype setBordered:NO];

		matrix=[[NSMatrix alloc]
				initWithFrame:NSMakeRect(field_left,*y,field_width,
										count*(lineHeight+2))
				mode:NSRadioModeMatrix
				prototype:buttonPrototype
				numberOfRows:count
				numberOfColumns:1];
		for(i=0;i<count;i++){		
			NSButtonCell* cell=[matrix cellAtRow:i column:0];
			[cell setTitle:[choices objectAtIndex:i]];
			[cell setTag:i];
		}
		[matrix setAutoresizingMask:NSViewWidthSizable|NSViewMinYMargin];
		[[calculatorWindow contentView]addSubview:matrix];
		if(action!=selector_none){
			[matrix setEnabled:YES];
			[matrix setTarget:self];
			[matrix setAction:action];
		}
		return(matrix);
	}//addMatrixNamed:at:action:choices:;


	-(void)buildWindow
	{
		float		window_width=field_right+10.0;
		float		y=350.0;
		NSButton*	compBut;

		calculatorWindow=[[NSWindow alloc]
				initWithContentRect:NSMakeRect(20.0,20.0,window_width,y)
				styleMask:NSTitledWindowMask|NSMiniaturizableWindowMask
				backing:NSBackingStoreBuffered
				defer:NO];
		[calculatorWindow setTitle:@"Untitled OO Estimate"];

		keyClassCount       =[self addTextFieldNamed:@"number key classes ="
							  at:&y action:@selector(setKeyClassCount:)];
		personCount			=[self addTextFieldNamed:@"number of persons ="
							  at:&y action:@selector(setPersonCount:)];
		personDayPerClass	=[self addTextFieldNamed:@"person day/class="
							  at:&y action:selector_none];
		experienceRatioScroll
					=[self addScrollerNamed:
					  @"ratio of experienced to novice OO personnel ="
					  at:&y action:@selector(setExperienceRatio:)];
		reusableDomainObjectsScroll
					=[self addScrollerNamed:
					  @"number of reusable domain objects ="
				      at:&y action:@selector(setReusableDomainObjects:)];
		userInterfaceComplexityRadios
					=[self addMatrixNamed:@"user interface complexity ="
					  at:&y action:@selector(setUserInterfaceComplexity:)
					  choices:[NSArray arrayWithObjects:
							@"simple",@"medium",@"complex",nil]];
									
		totalMonths     =[self addTextFieldNamed:@"total number of months="
						  at:&y action:selector_none];
    	totalPersonDays =[self addTextFieldNamed:@"total number of person.day="
						  at:&y action:selector_none];
		totalClassCount =[self addTextFieldNamed:@"total number of classes ="
						  at:&y action:selector_none];


		compBut=[[NSButton alloc] initWithFrame:
				NSMakeRect(label_left,10.0,field_right-label_left,24.0)];
		[compBut setTitle:@"Compute!"];
		[compBut setTarget:self];
		[compBut setAction:@selector(compute:)];
		[compBut setAutoresizingMask:NSViewWidthSizable|NSViewMaxYMargin];
		[[calculatorWindow contentView]addSubview:compBut];

	}//buildWindow


	-inhibitedPanel:sender
	{
		NSPanel* panel=NSGetAlertPanel(@"Inhibited function",
                   @"Sorry, this function has been inhibited because it "
							@"latches a critical bug in GNUstep.",
                   @"Too bad",
                   nil,nil);
		[NSApp runModalForWindow:panel];
		return(self);	
	}//inhibitedPanel:


	-(NSMenu*)makeMenu
	{
		NSMenuItem* item;
		NSMenu* menu=[[NSMenu alloc] initWithTitle:@"ooestimate"];
		
		item=[menu addItemWithTitle:@"Reset"
				   action:@selector(reset:)
				   keyEquivalent:nil];
		[item setTarget:self];
		[item setEnabled:YES];

		item=[menu addItemWithTitle:@"Load..."
				   action:selector_load
				   keyEquivalent:@"o"];
		[item setTarget:self];
		[item setEnabled:YES];

		item=[menu addItemWithTitle:@"Save"
				   action:selector_save
				   keyEquivalent:@"s"];
		[item setTarget:self];
		[item setEnabled:YES];


		item=[menu addItemWithTitle:@"Save as..."
				   action:selector_saveAs
				   keyEquivalent:@"S"];
		[item setTarget:self];
		[item setEnabled:YES];



		item=[menu addItemWithTitle:@"Hide"
				   action:@selector(hide:)
				   keyEquivalent:@"h"];
		[item setEnabled:YES];

		item=[menu addItemWithTitle:@"Quit"
				   action:@selector(terminate:)
				   keyEquivalent:@"q"];
		[item setEnabled:YES];

		[menu sizeToFit];
		return(menu);		
	}//makeMenu;


	- appDidInit:sender 
	{
		[self reset:sender];
		return self;
	}
	
	- resetData
	{
	   	[keyClassCount setIntValue:1];
   		[personCount setIntValue:1];
		[experienceRatioScroll setFloatValue:0.5];
		[reusableDomainObjectsScroll setFloatValue:0.5];
		[userInterfaceComplexityRadios selectCellWithTag:2];
		[self setExperienceRatio:experienceRatioScroll];
		[self setReusableDomainObjects:reusableDomainObjectsScroll];
		[self setUserInterfaceComplexity:userInterfaceComplexityRadios];
		[self compute:self];
		return self;
	}

	- windowDidBecomeMain:sender
	{
		static int first=1;
		if(first){
			first=0;
			[self resetData];
		}
		return self;
	}
	
	
	- setExperienceRatio:sender
	{
		experienceRatio=[sender floatValue];
		return [self compute:sender];
	}
	
	- setKeyClassCount:sender
	{
		keyClassCount=sender;
		return [self compute:sender];
	}
	
	- setPersonCount:sender
	{
		personCount=sender;
		return [self compute:sender];
	}
	
	- setReusableDomainObjects:sender
	{
		reusableDomainObjects=[sender floatValue];
		return [self compute:sender];
	}
	
	- setUserInterfaceComplexity:sender
	{
		static float uic[]={2.0,2.25,2.5,3.0};
		userInterfaceComplexity=uic[[[sender selectedCell] tag]];
		return [self compute:sender];
	}
	
	- (float)keyClassCount
	{
		return [keyClassCount floatValue];
	}
	
	- (float)personCount
	{
		float value=[personCount floatValue];
		if(value<0.00001){
			value=1.0;
			[personCount setFloatValue:value];
		}
		return value;
	}
	
	- (float)reusableDomainObjects
	{
		return reusableDomainObjects;
	}
	
	- (float)userInterfaceComplexity
	{
		return userInterfaceComplexity;
	}
	
	- (float)experienceRatio
	{
		return experienceRatio;
	}
	
	- compute:sender
	{
		float personDayPerClassValue=(15+2.5*(1-[self experienceRatio])
					+2.5*(1-[self reusableDomainObjects]));
		float totalClassCountValue=[self keyClassCount]
					*(1+[self userInterfaceComplexity]);
		float totalPersonDaysValue=totalClassCountValue*personDayPerClassValue;
		float totalMonthsValue=totalPersonDaysValue/20.0/[self personCount];
		[totalClassCount setFloatValue:(int)totalClassCountValue];
		[personDayPerClass setFloatValue:personDayPerClassValue];
		[totalPersonDays setFloatValue:totalPersonDaysValue];
		[totalMonths setFloatValue:totalMonthsValue];
		
		return self;
	}
	
	
	-(void)setFileName:(NSString*)fname
	{

		if(filename!=nil){
			[filename release];
		}
		if(fname!=nil){
			filename=[fname retain];
			[calculatorWindow setTitle:[NSString 
					stringWithFormat:@"%@ OO Estimation",filename]];
		}else{
			filename=nil;
			[calculatorWindow setTitle:@"Untitled OO Estimation"];
		}	
	}//setFileName:;
	
	
	- reset:sender
	{
		[self resetData];
		[self setFileName:nil];
 		return(self);
	}//reset:;
	

	typedef struct {
		int			keyClassCount;
		int			personCount;
		int			userInterfaceComplexity;
		float		experienceRatio;
		float		reusableDomainObjects;		
	}			state_t;
	

	-(BOOL)loadFromFile:(NSString*)fname
	{
			FILE*		in;
			state_t		buf;
			
		in=fopen([fname cString],"r");
		if(in!=NULL){
			if((1==fscanf(in,"%d",&(buf.keyClassCount)))
			&&(1==fscanf(in,"%d",&(buf.personCount)))
			&&(1==fscanf(in,"%d",&(buf.userInterfaceComplexity)))
			&&(1==fscanf(in,"%f",&(buf.experienceRatio)))
			&&(1==fscanf(in,"%f",&(buf.reusableDomainObjects)))){
				[keyClassCount setIntValue:buf.keyClassCount];
				[personCount setIntValue:buf.personCount];
				[experienceRatioScroll setFloatValue:buf.experienceRatio];
				[reusableDomainObjectsScroll 
						setFloatValue:buf.reusableDomainObjects];
				[userInterfaceComplexityRadios 
						selectCellWithTag:buf.userInterfaceComplexity];
				[self setExperienceRatio:experienceRatioScroll];
				[self setReusableDomainObjects:reusableDomainObjectsScroll];
				[self setUserInterfaceComplexity:
						userInterfaceComplexityRadios];
				[self compute:self];
				fclose(in);
				return(YES);
			}else{
				fclose(in);
				return(NO);
			}
		}else{
			return(NO);
		}
	}//loadFromFile:;
	
	
	-(BOOL)saveToFile:(NSString*)fname
	{
			state_t		buf;
			FILE*		out;
			
		buf.experienceRatio=[self experienceRatio];
		buf.reusableDomainObjects=[self reusableDomainObjects];
		buf.keyClassCount=[self keyClassCount];
		buf.userInterfaceComplexity=[self userInterfaceComplexity];
		buf.personCount=[self personCount];
		out=fopen([fname cString],"w+");
		if(out!=NULL){
			fprintf(out,"%d %d %d %f %f\n",buf.keyClassCount,
					buf.personCount,buf.userInterfaceComplexity,
					buf.experienceRatio,buf.reusableDomainObjects);
			fprintf(out,"keyClassCount personCount "
					"userInterfaceComplexity experienceRatio "
					"reusableDomainObjects\n");
			fclose(out);
			return(YES);
		}else{
			return(NO);
		}
	}//saveToFile:;
	
	
	
	- load:sender
	{
			static NSOpenPanel*	openPanel=nil;
			static NSArray*		filetypes=nil;
			NSArray*			files;
			NSString*			newfilename;
		if(openPanel==nil){
			openPanel=[NSOpenPanel openPanel];
		}
		if(filetypes==nil){
			filetypes=[NSArray arrayWithObjects:@"ooestimate",nil];
		}
		if([openPanel runModalForTypes:filetypes]==NSOKButton){
			files=[openPanel filenames];
			newfilename=[files objectAtIndex:0];
			if([self loadFromFile:newfilename]){
				[self setFileName:newfilename];
			}
		}
		return(self);	
	}//load:;
	
	
	- save:sender
	{
		if(filename==NULL){
			[self saveAs:sender];
		}else{
			[self saveToFile:filename];
		}
		return(self);	
	}//save:;
	
	
	- saveAs:sender
	{
			static NSSavePanel*	savePanel=nil;
			NSString*			newfilename;
		if(savePanel==nil){
			savePanel=[NSSavePanel new];
		}
		[savePanel setRequiredFileType:@"ooestimate"];
		if([savePanel runModal]==NSOKButton){
			newfilename=[savePanel filename];
			if([self saveToFile:newfilename]){
				[self setFileName:newfilename];
			}
		}
		return(self);
	}//saveAs:;
	
	

@end
