
#import "Calculator.h"

@implementation Calculator

	- init
	{
		[NXApp setDelegate:self];
		filename=NULL;
		return self;
	}
	
	- appDidInit:sender 
	{
		[calculatorWindow setDelegate:self];
		[self reset:sender];
		return self;
	}
	
	- resetData
	{
	   	[keyClassCount setIntValue:1];
   		[personCount setIntValue:1];
		[experienceRatioScroll setFloatValue:0.5];
		[reusableDomainObjectsScroll setFloatValue:0.5];
		[userInterfaceComplexityRadios selectCellWithTag:2];
		[self setExperienceRatio:experienceRatioScroll];
		[self setReusableDomainObjects:reusableDomainObjectsScroll];
		[self setUserInterfaceComplexity:userInterfaceComplexityRadios];
		[self compute:self];
		return self;
	}

	- windowDidBecomeMain:sender
	{
		static int first=1;
		if(first){
			first=0;
			[self resetData];
		}
		return self;
	}
	
	
	- setExperienceRatio:sender
	{
		experienceRatio=[sender floatValue];
		return [self compute:sender];
	}
	
	- setKeyClassCount:sender
	{
		keyClassCount=sender;
		return [self compute:sender];
	}
	
	- setPersonCount:sender
	{
		personCount=sender;
		return [self compute:sender];
	}
	
	- setReusableDomainObjects:sender
	{
		reusableDomainObjects=[sender floatValue];
		return [self compute:sender];
	}
	
	- setUserInterfaceComplexity:sender
	{
			static float uic[]={2.0,2.25,2.5,3.0};
		userInterfaceComplexity=uic[[[sender selectedCell] tag]];
		return [self compute:sender];
	}
	
	- (float)keyClassCount
	{
		return [keyClassCount floatValue];
	}
	
	- (float)personCount
	{
		float value=[personCount floatValue];
		if(value<0.00001){
			value=1.0;
			[personCount setFloatValue:value];
		}
		return value;
	}
	
	- (float)reusableDomainObjects
	{
		return reusableDomainObjects;
	}
	
	- (float)userInterfaceComplexity
	{
		return userInterfaceComplexity;
	}
	
	- (float)experienceRatio
	{
		return experienceRatio;
	}
	
	- compute:sender
	{
		float personDayPerClassValue=(15+2.5*(1-[self experienceRatio])
					+2.5*(1-[self reusableDomainObjects]));
		float totalClassCountValue=[self keyClassCount]
					*(1+[self userInterfaceComplexity]);
		float totalPersonDaysValue=totalClassCountValue*personDayPerClassValue;
		float totalMonthsValue=totalPersonDaysValue/20.0/[self personCount];
		[totalClassCount setFloatValue:(int)totalClassCountValue];
		[personDayPerClass setFloatValue:personDayPerClassValue];
		[totalPersonDays setFloatValue:totalPersonDaysValue];
		[totalMonths setFloatValue:totalMonthsValue];
		
		return self;
	}
	
	
	-(void)setFileName:(const char*)fname
	{
			char*	buffer;
		if(filename!=NULL){
			free(filename);
		}
		if(fname!=NULL){
			filename=(char*)malloc(strlen(fname)+1);
			strcpy(filename,fname);
			buffer=(char*)malloc(strlen(filename)+48);
			sprintf(buffer,"%s OO Estimation",filename);
			[calculatorWindow setTitle:buffer];
			free(buffer);   
		}else{
			filename=NULL;
			[calculatorWindow setTitle:"Untitled OO Estimation"];
		}	
	}//setFileName:;
	
	
	- reset:sender
	{
		[self resetData];
		[self setFileName:NULL];
 		return(self);
	}//reset:;
	

	typedef struct {
		int			keyClassCount;
		int			personCount;
		int			userInterfaceComplexity;
		float		experienceRatio;
		float		reusableDomainObjects;		
	}			state_t;
	

	-(BOOL)loadFromFile:(const char*)fname
	{
			FILE*		in;
			state_t		buf;
			
		in=fopen(fname,"r");
		if(in!=NULL){
			if((1==fscanf(in,"%d",&(buf.keyClassCount)))
			&&(1==fscanf(in,"%d",&(buf.personCount)))
			&&(1==fscanf(in,"%d",&(buf.userInterfaceComplexity)))
			&&(1==fscanf(in,"%f",&(buf.experienceRatio)))
			&&(1==fscanf(in,"%f",&(buf.reusableDomainObjects)))){
				[keyClassCount setIntValue:buf.keyClassCount];
				[personCount setIntValue:buf.personCount];
				[experienceRatioScroll setFloatValue:buf.experienceRatio];
				[reusableDomainObjectsScroll 
						setFloatValue:buf.reusableDomainObjects];
				[userInterfaceComplexityRadios 
						selectCellWithTag:buf.userInterfaceComplexity];
				[self setExperienceRatio:experienceRatioScroll];
				[self setReusableDomainObjects:reusableDomainObjectsScroll];
				[self setUserInterfaceComplexity:
						userInterfaceComplexityRadios];
				[self compute:self];
				fclose(in);
				return(YES);
			}else{
				fclose(in);
				return(NO);
			}
		}else{
			return(NO);
		}
	}//loadFromFile:;
	
	
	-(BOOL)saveToFile:(const char*)fname
	{
			state_t		buf;
			FILE*		out;
			
		buf.experienceRatio=[self experienceRatio];
		buf.reusableDomainObjects=[self reusableDomainObjects];
		buf.keyClassCount=[self keyClassCount];
		buf.userInterfaceComplexity=[self userInterfaceComplexity];
		buf.personCount=[self personCount];
		out=fopen(fname,"w+");
		if(out!=NULL){
			fprintf(out,"%d %d %d %f %f\n",buf.keyClassCount,
					buf.personCount,buf.userInterfaceComplexity,
					buf.experienceRatio,buf.reusableDomainObjects);
			fprintf(out,"keyClassCount personCount "
					"userInterfaceComplexity experienceRatio "
					"reusableDomainObjects\n");
			fclose(out);
			return(YES);
		}else{
			return(NO);
		}
	}//saveToFile:;
	
	
	
	- load:sender
	{
			static OpenPanel*	openPanel=nil;
			static char*		filetypes[]={"ooestimate",NULL};
			const char*const*	files;
			const char*			newfilename;
		if(openPanel==nil){
			openPanel=[OpenPanel new];
		}
		if([openPanel runModalForTypes:filetypes]==NX_OKTAG){
			files=[openPanel filenames];
			newfilename=files[0];
			if([self loadFromFile:newfilename]){
				[self setFileName:newfilename];
			}
		}
		return(self);	
	}//load:;
	
	
	- save:sender
	{
		if(filename==NULL){
			[self saveAs:sender];
		}else{
			[self saveToFile:filename];
		}
		return(self);	
	}//save:;
	
	
	- saveAs:sender
	{
			static SavePanel*	savePanel=nil;
			const char*			newfilename;
		if(savePanel==nil){
			savePanel=[SavePanel new];
		}
		[savePanel setRequiredFileType:"ooestimate"];
		if([savePanel runModal]==NX_OKTAG){
			newfilename=[savePanel filename];
			if([self saveToFile:newfilename]){
				[self setFileName:newfilename];
			}
		}
		return(self);
	}//saveAs:;
	
	

@end
